import setuptools

    
setuptools.setup(
    name = "IOTAILibrary",
    version = "0.0.0.7",
    author = "TimmyHuang",
    author_email="timmy.huang.iot@gmail.com",
    description="IOTAILibrary",
    long_description="IOT Library For AI Training",
    long_description_content_type="text/markdown",
    url="https://github.com/HuangTeEn",                                         
    packages=setuptools.find_packages(),     
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6'
    )
    
    